'use strict';

exports.__esModule = true;
exports.default = createValidatorWithMultipleErrors;

var _isValueValidator = require('./isValueValidator');

var _isValueValidator2 = _interopRequireDefault(_isValueValidator);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function buildErrorsArray(validators, validate) {
  return validators.reduce(function (errors, validator) {
    var errorMessage = validate(validator);

    if (errorMessage) {
      errors.push(errorMessage);
    }

    return errors;
  }, []);
}

function buildErrorsObject(validators, validate) {
  return Object.keys(validators).reduce(function (errors, key) {
    var validator = validators[key];
    var errorMessage = validate(validator);

    if (errorMessage) {
      errors[key] = errorMessage;
    }

    return errors;
  }, {});
}

function createValidatorWithMultipleErrors(validators, sharedConfig) {
  var buildErrors = void 0;
  var finalValidators = void 0;

  if (typeof validators[0] === 'object') {
    buildErrors = buildErrorsObject;
    finalValidators = validators[0];
  } else {
    buildErrors = buildErrorsArray;
    finalValidators = validators;
  }

  return function composedValidator(value, allValues) {
    return buildErrors(finalValidators, function (validator) {
      if ((0, _isValueValidator2.default)(validator)) {
        return validator(value, allValues);
      }

      return validator(sharedConfig, value, allValues);
    });
  };
}